@@ -2,13 +2,10 @@ package Amon2::Web::Dispatcher::RouterSimple::Extended;
 use strict;
 use warnings;
 
-our $VERSION = "0.05.01";
+our $VERSION = "0.01";
 
 use Router::Simple 0.03;
 
-my @METHODS = qw/GET POST PUT DELETE/;
-my $submap;
-
 sub import {
     my $class = shift;
     my %args = @_;
@@ -16,54 +13,8 @@ sub import {
 
     my $router = Router::Simple->new;
 
-    no strict 'refs';
-
-    # functions
-    *{"${caller}::submapper"} = \&_submapper;
-    *{"${caller}::connect"} = \&_connect;
-    for my $method (@METHODS) {
-        *{"${caller}::@{[lc $method]}"} = _make_method_connector($caller, $method);
-    }
-
-    # class methods
-    *{"${caller}::router"} = sub { $router };
-    for my $meth (qw/match as_string/) {
-        *{"$caller\::${meth}"} = sub {
-            my $class = shift;
-            $class->router->$meth(@_)
-        };
-    }
-    *{"$caller\::dispatch"} = \&_dispatch;
-}
-
-sub _make_method_connector {
-    my ($caller, $method) = @_;
-
-    sub {
-        my $class = caller(0);
-
-        if ($submap) {
-            my ($path, $action) = @_;
-            $submap->connect($path, { action => $action }, { metod => $method });
-        } else {
-            $_[2] = { method => $method };
-            goto \&_connect;
-        }
-    }
-}
-
-sub _connect {
-    my $caller = caller(0);
-    if ($submap) {
-        if (@_ >= 2 && !ref $_[1]) {
-            my ($path, $action, $opt) = @_;
-            $submap->connect($path, { action => $action }, $opt || {});
-        } else {
-            $submap->connect(@_);
-        }
-    } else {
-        my $router = $caller->router;
-        if (@_ >= 2 && !ref $_[1]) {
+    my $connect = sub {
+        if (@_ == 2 && !ref $_[1]) {
             my ($path, $dest_str, $opt) = @_;
             my ($controller, $action) = split('#', $dest_str);
             my $dest = { controller => $controller };
@@ -72,21 +23,68 @@ sub _connect {
         } else {
             $router->connect(@_);
         }
-    }
-}
+    };
 
-sub _submapper {
-    my $caller = caller(0);
-    my $router = caller(0)->router();
-    if ($_[2] && ref($_[2]) eq 'CODE') {
-        my ($path, $controller, $callback) = @_;
-        $submap = $router->submapper($path, { controller => $controller });
-        $callback->();
-        undef $submap;
+    no strict 'refs';
+
+    # functions
+    *{"${caller}::connect"} = $connect;
+    my @methods = qw/GET POST PUT DELETE/;
+    my %procs;
+    for my $method (@methods) {
+        *{"${caller}::@{[lc $method]}"} = $procs{$method} = sub {
+            $connect->($_[0], $_[1], { method => $method });
+        };
     }
-    else {
-        $router->submapper(@_);
+
+    use strict 'refs';
+
+    my $submapper = sub {
+        if ($_[2] && ref($_[2]) eq 'CODE') {
+            my ($path, $controller, $callback) = @_;
+            my $submap = $router->submapper($path, { controller => $controller });
+            my $new_connect = sub {
+                if (@_ >= 2 && !ref $_[1]) {
+                    my ($path, $action, $opt) = @_;
+                    $submap->connect($path, { action => $action }, $opt || {});
+                } else {
+                    $submap->connect(@_);
+                }
+            };
+            no strict 'refs';
+            no warnings 'redefine';
+            *{"${caller}::connect"} = $new_connect;
+            for my $method (@methods) {
+                *{"${caller}::@{[lc $method]}"} = sub {
+                    my ($path, $action) = @_;
+                    $submap->connect($path, { action => $action }, { metod => $method });
+                };
+            }
+            use strict 'refs';
+            use warnings 'redefine';
+            $callback->();
+            no strict 'refs';
+            no warnings 'redefine';
+            *{"${caller}::connect"} = $connect;
+            *{"${caller}::@{[ lc $_ ]}"} = $procs{$_} for (@methods);
+        }
+        else {
+            $router->submapper(@_);
+        }
+    };
+
+    no strict 'refs';
+
+    *{"${caller}::submapper"} = $submapper;
+    # class methods
+    *{"${caller}::router"} = sub { $router };
+    for my $meth (qw/match as_string/) {
+        *{"$caller\::${meth}"} = sub {
+            my $self = shift;
+            $router->$meth(@_)
+        };
     }
+    *{"$caller\::dispatch"} = \&_dispatch;
 }
 
 sub _dispatch {
@@ -144,31 +142,19 @@ This is an extension of Amon2::Web::Dispatcher::RouterSimple. 100% compatible, a
 
 =item get $path, "${controller}#${action}"
 
-this is equivalent to:
-
-    connect $path, { controller => $controller, action => $action }, { method => 'GET' };
-
+this is equivalent to 'connect $path, { controller => $controller, action => $action }, { method => 'GET' };'
 
 =item post $path, "${controller}#${action}"
 
-this is equivalent to:
-
-    connect $path, { controller => $controller, action => $action }, { method => 'POST' };
-
+this is equivalent to 'connect $path, { controller => $controller, action => $action }, { method => 'POST' };'
 
 =item put $path, "${controller}#${action}"
 
-this is equivalent to:
-
-    connect $path, { controller => $controller, action => $action }, { method => 'PUT' };
-
+this is equivalent to 'connect $path, { controller => $controller, action => $action }, { method => 'PUT' };'
 
 =item delete $path, "${controller}#${action}"
 
-this is equivalent to:
-
-    connect $path, { controller => $controller, action => $action }, { method => 'DELETE' };
-
+this is equivalent to 'connect $path, { controller => $controller, action => $action }, { method => 'DELETE' };'
 
 =item submapper $path, $controller, sub {}
 
@@ -179,15 +165,15 @@ this is main feature of this module. In subroutine of the third argument, connec
 
 =head1 LICENSE
 
-Copyright (C) Taiyoh Tanaka
+Copyright (C) taiyoh
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =head1 AUTHOR
 
-Taiyoh Tanaka <sun.basix@gmail.com>
+taiyoh
 
 =head1 SEE ALSO
 
-L<Amon2::Web::Dispatcher::RouterSimple|http://search.cpan.org/~tokuhirom/Amon2/lib/Amon2/Web/Dispatcher/RouterSimple.pm>
+L<Amon2::Web::Dispatcher::RouterSimple>
